/** 
 * DevExtreme (integration/knockout/component_registrator.js)
 * Version: 16.1.6
 * Build date: Fri Sep 02 2016
 *
 * Copyright (c) 2012 - 2016 Developer Express Inc. ALL RIGHTS RESERVED
 * EULA: https://www.devexpress.com/Support/EULAs/DevExtreme.xml
 * MAY BE USED WITH: DevExtreme Complete, DevExtreme Mobile, DevExtreme Web
 */
"use strict";
define(function(require, exports, module) {
    var $ = require("jquery"),
        ko = require("knockout"),
        registerComponent = require("../../core/component_registrator"),
        Widget = require("../../ui/widget/ui.widget"),
        KoTemplateProvider = require("./template_provider"),
        Editor = require("../../ui/editor/editor"),
        Locker = require("../../core/utils/locker");
    var LOCKS_DATA_KEY = "dxKoLocks",
        CREATED_WITH_KO_DATA_KEY = "dxKoCreation";
    var editorsBindingHandlers = [];
    var registerComponentKoBinding = function(componentName, componentClass) {
        if (componentClass.subclassOf(Editor)) {
            editorsBindingHandlers.push(componentName)
        }
        ko.bindingHandlers[componentName] = {
            init: function(domNode, valueAccessor) {
                var $element = $(domNode),
                    optionChangedCallbacks = $.Callbacks(),
                    ctorOptions = {
                        templateProvider: KoTemplateProvider,
                        modelByElement: function($element) {
                            if ($element.length) {
                                return ko.dataFor($element.get(0))
                            }
                        },
                        nestedComponentOptions: function(component) {
                            return {
                                modelByElement: component.option("modelByElement"),
                                nestedComponentOptions: component.option("nestedComponentOptions")
                            }
                        },
                        watchMethod: function(watchValue, callback, element) {
                            var values;
                            ko.computed(function() {
                                if (values) {
                                    callback()
                                }
                                values = watchValue()
                            }, null, {
                                disposeWhenNodeIsRemoved: element
                            })
                        },
                        _optionChangedCallbacks: optionChangedCallbacks
                    },
                    optionNameToModelMap = {};
                var applyModelValueToOption = function(optionName, modelValue) {
                    var component = $element.data(componentName),
                        locks = $element.data(LOCKS_DATA_KEY),
                        optionValue = ko.unwrap(modelValue);
                    if (ko.isWriteableObservable(modelValue)) {
                        optionNameToModelMap[optionName] = modelValue
                    }
                    if (component) {
                        if (locks.locked(optionName)) {
                            return
                        }
                        locks.obtain(optionName);
                        try {
                            if (ko.ignoreDependencies) {
                                ko.ignoreDependencies(component.option, component, [optionName, optionValue])
                            } else {
                                component.option(optionName, optionValue)
                            }
                        } finally {
                            locks.release(optionName)
                        }
                    } else {
                        ctorOptions[optionName] = optionValue
                    }
                };
                var handleOptionChanged = function(args) {
                    var optionName = args.fullName,
                        optionValue = args.value;
                    if (!(optionName in optionNameToModelMap)) {
                        return
                    }
                    var $element = this._$element,
                        locks = $element.data(LOCKS_DATA_KEY);
                    if (locks.locked(optionName)) {
                        return
                    }
                    locks.obtain(optionName);
                    try {
                        optionNameToModelMap[optionName](optionValue)
                    } finally {
                        locks.release(optionName)
                    }
                };
                var createComponent = function() {
                    optionChangedCallbacks.add(handleOptionChanged);
                    $element.data(CREATED_WITH_KO_DATA_KEY, true).data(LOCKS_DATA_KEY, new Locker)[componentName](ctorOptions);
                    ctorOptions = null
                };
                var unwrapModelValue = function(currentModel, propertyName, propertyPath) {
                    var unwrappedPropertyValue;
                    ko.computed(function() {
                        var propertyValue = currentModel[propertyName];
                        applyModelValueToOption(propertyPath, propertyValue);
                        unwrappedPropertyValue = ko.unwrap(propertyValue)
                    }, null, {
                        disposeWhenNodeIsRemoved: domNode
                    });
                    if ($.isPlainObject(unwrappedPropertyValue)) {
                        unwrapModel(unwrappedPropertyValue, propertyPath)
                    }
                };
                var unwrapModel = function(model, propertyPath) {
                    for (var propertyName in model) {
                        if (model.hasOwnProperty(propertyName)) {
                            unwrapModelValue(model, propertyName, propertyPath ? [propertyPath, propertyName].join(".") : propertyName)
                        }
                    }
                };
                ko.computed(function() {
                    var component = $element.data(componentName),
                        model = ko.unwrap(valueAccessor());
                    if (component) {
                        component.beginUpdate()
                    }
                    unwrapModel(model);
                    if (component) {
                        component.endUpdate()
                    } else {
                        createComponent()
                    }
                }, null, {
                    disposeWhenNodeIsRemoved: domNode
                });
                return {
                    controlsDescendantBindings: componentClass.subclassOf(Widget)
                }
            }
        };
        if ("dxValidator" === componentName) {
            ko.bindingHandlers.dxValidator.after = editorsBindingHandlers
        }
    };
    registerComponent.callbacks.add(function(name, componentClass) {
        registerComponentKoBinding(name, componentClass)
    })
});
